@@ -1,38 +1,37 @@
 """Shared config dialog stuff"""
-# Standard Library
-# pylint: disable=no-member,not-an-iterable
-import importlib
+# pylint: disable=not-an-iterable
 import os
 from gettext import gettext as _
 
-# Third Party Libraries
 from gi.repository import Gdk, GLib, Gtk, Pango
 
-# Lutris Modules
 from lutris import runners, settings
-from lutris.cache import get_cache_path, save_cache_path
 from lutris.config import LutrisConfig, make_game_config_id
 from lutris.game import Game
+from lutris.gui import dialogs
+from lutris.gui.config import DIALOG_HEIGHT, DIALOG_WIDTH
 from lutris.gui.config.boxes import GameBox, RunnerBox, SystemBox
-from lutris.gui.dialogs import ErrorDialog, QuestionDialog
-from lutris.gui.widgets.common import FileChooserEntry, Label, NumberEntry, SlugEntry, VBox
-from lutris.gui.widgets.log_text_view import LogTextView
-from lutris.gui.widgets.utils import BANNER_SIZE, ICON_SIZE, get_pixbuf, get_pixbuf_for_game
-from lutris.util import resources
-from lutris.util.linux import gather_system_info_str
+from lutris.gui.dialogs import Dialog, DirectoryDialog, ErrorDialog, QuestionDialog
+from lutris.gui.widgets.common import Label, NumberEntry, SlugEntry, VBox
+from lutris.gui.widgets.notifications import send_notification
+from lutris.gui.widgets.utils import BANNER_SIZE, ICON_SIZE, get_pixbuf
+from lutris.runners import import_runner
+from lutris.services.lutris import LutrisBanner, LutrisIcon
+from lutris.util import resources, system
 from lutris.util.log import logger
 from lutris.util.strings import slugify
 
 
-# pylint: disable=too-many-instance-attributes
-class GameDialogCommon:
-
-    """Mixin for config dialogs"""
+# pylint: disable=too-many-instance-attributes, no-member
+class GameDialogCommon(Dialog):
+    """Base class for config dialogs"""
     no_runner_label = _("Select a runner in the Game Info tab")
 
-    def __init__(self):
+    def __init__(self, title, parent=None):
+        super().__init__(title, parent=parent)
+        self.set_type_hint(Gdk.WindowTypeHint.NORMAL)
+        self.set_default_size(DIALOG_WIDTH, DIALOG_HEIGHT)
         self.notebook = None
-        self.vbox = None
         self.name_entry = None
         self.runner_box = None
 
@@ -41,6 +40,7 @@ class GameDialogCommon:
         self.saved = None
         self.slug = None
         self.slug_entry = None
+        self.directory_entry = None
         self.year_entry = None
         self.slug_change_button = None
         self.runner_dropdown = None
@@ -48,35 +48,44 @@ class GameDialogCommon:
         self.icon_button = None
         self.game_box = None
         self.system_box = None
-        self.system_sw = None
         self.runner_name = None
         self.runner_index = None
         self.lutris_config = None
-        self.clipboard = None
-        self._clipboard_buffer = None
+
+        # These are independent windows, but start centered over
+        # a parent like a dialog. Not modal, not really transient,
+        # and does not share modality with other windows - so it
+        # needs its own window group.
+        Gtk.WindowGroup().add_window(self)
+        GLib.idle_add(self.clear_transient_for)
+
+    def clear_transient_for(self):
+        # we need the parent set to be centered over the parent, but
+        # we don't want to be transient really- we want other windows
+        # able to come to the front.
+        self.set_transient_for(None)
+        return False
 
     @staticmethod
     def build_scrolled_window(widget):
-        """Return a scrolled window for containing config widgets"""
-        scrolled_window = Gtk.ScrolledWindow()
+        """Return a scrolled window containing config widgets"""
+        scrolled_window = Gtk.ScrolledWindow(visible=True)
         scrolled_window.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
         scrolled_window.add(widget)
         return scrolled_window
 
     def build_notebook(self):
-        self.notebook = Gtk.Notebook()
+        self.notebook = Gtk.Notebook(visible=True)
+        self.notebook.set_show_border(False)
         self.vbox.pack_start(self.notebook, True, True, 10)
 
     def build_tabs(self, config_level):
+        """Build tabs (for game and runner levels)"""
         self.timer_id = None
         if config_level == "game":
             self._build_info_tab()
             self._build_game_tab()
-        if config_level in ("game", "runner"):
-            self._build_runner_tab(config_level)
-        if config_level == "system":
-            self._build_prefs_tab()
-            self._build_sysinfo_tab()
+        self._build_runner_tab(config_level)
         self._build_system_tab(config_level)
 
     def _build_info_tab(self):
@@ -87,98 +96,24 @@ class GameDialogCommon:
 
         info_box.pack_start(self._get_name_box(), False, False, 6)  # Game name
 
-        if self.game:
-            info_box.pack_start(self._get_slug_box(), False, False, 6)  # Game id
-
         self.runner_box = self._get_runner_box()
         info_box.pack_start(self.runner_box, False, False, 6)  # Runner
 
         info_box.pack_start(self._get_year_box(), False, False, 6)  # Year
 
+        if self.game:
+            info_box.pack_start(self._get_slug_box(), False, False, 6)
+            info_box.pack_start(self._get_directory_box(), False, False, 6)
+
         info_sw = self.build_scrolled_window(info_box)
         self._add_notebook_tab(info_sw, _("Game info"))
 
-    def _build_prefs_tab(self):
-        prefs_box = VBox()
-        prefs_box.pack_start(self._get_game_cache_box(), False, False, 6)
-
-        cache_help_label = Gtk.Label(visible=True)
-        cache_help_label.set_size_request(400, -1)
-        cache_help_label.set_markup(_(
-            "If provided, this location will be used by installers to cache "
-            "downloaded files locally for future re-use. \nIf left empty, the "
-            "installer files are discarded after the install completion."
-        ))
-        prefs_box.pack_start(cache_help_label, False, False, 6)
-
-        prefs_box.pack_start(self._get_hide_on_game_launch_box(), False, False, 6)
-
-        info_sw = self.build_scrolled_window(prefs_box)
-        self._add_notebook_tab(info_sw, _("Lutris preferences"))
-
-    def _build_sysinfo_tab(self):
-        sysinfo_box = Gtk.VBox()
-        sysinfo_view = LogTextView()
-        sysinfo_view.set_cursor_visible(False)
-        sysinfo_str = gather_system_info_str()
-
-        text_buffer = sysinfo_view.get_buffer()
-        text_buffer.set_text(sysinfo_str)
-        self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
-        self._clipboard_buffer = sysinfo_str
-
-        button_copy = Gtk.Button(_("Copy System Info"))
-        button_copy.connect("clicked", self._copy_text)
-
-        sysinfo_box.add(sysinfo_view)
-        sysinfo_box.add(button_copy)
-        info_sw = self.build_scrolled_window(sysinfo_box)
-        self._add_notebook_tab(info_sw, _("System Information"))
-
-    def _copy_text(self, widget):  # pylint: disable=unused-argument
-        self.clipboard.set_text(self._clipboard_buffer, -1)
-
-    def _get_game_cache_box(self):
-        box = Gtk.Box(spacing=12, margin_right=12, margin_left=12)
-        label = Label(_("Cache path"))
-        box.pack_start(label, False, False, 0)
-        cache_path = get_cache_path()
-        path_chooser = FileChooserEntry(
-            title=_("Set the folder for the cache path"), action=Gtk.FileChooserAction.SELECT_FOLDER, path=cache_path
-        )
-        path_chooser.entry.connect("changed", self._on_cache_path_set)
-        box.pack_start(path_chooser, True, True, 0)
-        return box
-
-    def _get_hide_on_game_launch_box(self):
-        box = Gtk.Box(spacing=12, margin_right=12, margin_left=12)
-        checkbox = Gtk.CheckButton(label=_("Minimize client when a game is launched"))
-        if settings.read_setting("hide_client_on_game_start") == "True":
-            checkbox.set_active(True)
-        checkbox.connect("toggled", self._on_hide_client_change)
-        box.pack_start(checkbox, True, True, 0)
-        return box
-
-    def _on_hide_client_change(self, widget):
-        """Save setting for hiding the game on game launch"""
-        settings.write_setting("hide_client_on_game_start", widget.get_active())
-
-    def _on_cache_path_set(self, entry):
-        if self.timer_id:
-            GLib.source_remove(self.timer_id)
-        self.timer_id = GLib.timeout_add(1000, self.save_cache_setting, entry.get_text())
-
-    def save_cache_setting(self, value):
-        save_cache_path(value)
-        GLib.source_remove(self.timer_id)
-        self.timer_id = None
-        return False
-
     def _get_name_box(self):
         box = Gtk.Box(spacing=12, margin_right=12, margin_left=12)
         label = Label(_("Name"))
         box.pack_start(label, False, False, 0)
         self.name_entry = Gtk.Entry()
+        self.name_entry.set_max_length(150)
         if self.game:
             self.name_entry.set_text(self.game.name)
         box.pack_start(self.name_entry, True, True, 0)
@@ -202,6 +137,20 @@ class GameDialogCommon:
 
         return slug_box
 
+    def _get_directory_box(self):
+        """Return widget displaying the location of the game and allowing to move it"""
+        box = Gtk.Box(spacing=12, margin_right=12, margin_left=12, visible=True)
+        label = Label(_("Directory"))
+        box.pack_start(label, False, False, 0)
+        self.directory_entry = Gtk.Entry(visible=True)
+        self.directory_entry.set_text(self.game.directory)
+        self.directory_entry.set_sensitive(False)
+        box.pack_start(self.directory_entry, True, True, 0)
+        move_button = Gtk.Button(_("Move"), visible=True)
+        move_button.connect("clicked", self.on_move_clicked)
+        box.pack_start(move_button, False, False, 0)
+        return box
+
     def _get_runner_box(self):
         runner_box = Gtk.Box(spacing=12, margin_right=12, margin_left=12)
 
@@ -211,10 +160,6 @@ class GameDialogCommon:
         self.runner_dropdown = self._get_runner_dropdown()
         runner_box.pack_start(self.runner_dropdown, True, True, 0)
 
-        install_runners_btn = Gtk.Button(_("Install runners"))
-        install_runners_btn.connect("clicked", self.on_install_runners_clicked)
-        runner_box.pack_start(install_runners_btn, True, True, 0)
-
         return runner_box
 
     def _get_banner_box(self):
@@ -252,8 +197,8 @@ class GameDialogCommon:
 
         label = Label(_("Release year"))
         box.pack_start(label, False, False, 0)
-
         self.year_entry = NumberEntry()
+        self.year_entry.set_max_length(10)
         if self.game:
             self.year_entry.set_text(str(self.game.year or ""))
         box.pack_start(self.year_entry, True, True, 0)
@@ -262,19 +207,14 @@ class GameDialogCommon:
 
     def _set_image(self, image_format):
         image = Gtk.Image()
+        service_media = LutrisBanner() if image_format == "banner" else LutrisIcon()
         game_slug = self.game.slug if self.game else ""
-        image.set_from_pixbuf(get_pixbuf_for_game(game_slug, image_format))
+        image.set_from_pixbuf(service_media.get_pixbuf_for_game(game_slug))
         if image_format == "banner":
             self.banner_button.set_image(image)
         else:
             self.icon_button.set_image(image)
 
-    def _set_icon_image(self):
-        image = Gtk.Image()
-        game_slug = self.game.slug if self.game else ""
-        image.set_from_pixbuf(get_pixbuf_for_game(game_slug, "banner"))
-        self.banner_button.set_image(image)
-
     def _get_runner_dropdown(self):
         runner_liststore = self._get_runner_liststore()
         runner_dropdown = Gtk.ComboBox.new_with_model(runner_liststore)
@@ -319,17 +259,23 @@ class GameDialogCommon:
         self.slug_entry.set_sensitive(False)
         self.slug_change_button.set_label(_("Change"))
 
-    def on_install_runners_clicked(self, _button):
-        """Messed up callback requiring an import in the method to avoid a circular dependency"""
-        from lutris.gui.dialogs.runners import RunnersDialog
-        runners_dialog = RunnersDialog()
-        runners_dialog.connect("runner-installed", self.on_runner_installed)
+    def on_move_clicked(self, _button):
+        new_location = DirectoryDialog("Select new location for the game",
+                                       default_path=self.game.directory, parent=self)
+        if not new_location.folder or new_location.folder == self.game.directory:
+            return
+        move_dialog = dialogs.MoveDialog(self.game, new_location.folder)
+        move_dialog.connect("game-moved", self.on_game_moved)
+        move_dialog.move()
 
-    def on_runner_installed(self, _dialog):
-        """Callback triggered when new runners are installed"""
-        active_id = self.runner_dropdown.get_active_id()
-        self.runner_dropdown.set_model(self._get_runner_liststore())
-        self.runner_dropdown.set_active_id(active_id)
+    def on_game_moved(self, dialog):
+        """Show a notification when the game is moved"""
+        new_directory = dialog.new_directory
+        if new_directory:
+            self.directory_entry.set_text(new_directory)
+            send_notification("Finished moving game", "%s moved to %s" % (dialog.game, new_directory))
+        else:
+            send_notification("Failed to move game", "Lutris could not move %s" % dialog.game)
 
     def _build_game_tab(self):
         if self.game and self.runner_name:
@@ -362,8 +308,10 @@ class GameDialogCommon:
         if not self.lutris_config:
             raise RuntimeError("Lutris config not loaded yet")
         self.system_box = SystemBox(self.lutris_config)
-        self.system_sw = self.build_scrolled_window(self.system_box)
-        self._add_notebook_tab(self.system_sw, _("System options"))
+        self._add_notebook_tab(
+            self.build_scrolled_window(self.system_box),
+            _("System options")
+        )
 
     def _add_notebook_tab(self, widget, label):
         self.notebook.append_page(widget, Gtk.Label(label=label))
@@ -416,6 +364,7 @@ class GameDialogCommon:
         if self.runner_index and new_runner_index != self.runner_index:
             dlg = QuestionDialog(
                 {
+                    "parent": self,
                     "question":
                     _("Are you sure you want to change the runner for this game ? "
                       "This will reset the full configuration for this game and "
@@ -469,17 +418,16 @@ class GameDialogCommon:
 
     def is_valid(self):
         if not self.runner_name:
-            ErrorDialog(_("Runner not provided"))
+            ErrorDialog(_("Runner not provided"), parent=self)
             return False
         if not self.name_entry.get_text():
-            ErrorDialog(_("Please fill in the name"))
+            ErrorDialog(_("Please fill in the name"), parent=self)
             return False
-        if (self.runner_name in ("steam", "winesteam") and self.lutris_config.game_config.get("appid") is None):
-            ErrorDialog(_("Steam AppId not provided"))
+        if self.runner_name == "steam" and not self.lutris_config.game_config.get("appid"):
+            ErrorDialog(_("Steam AppID not provided"), parent=self)
             return False
         invalid_fields = []
-        runner_module = importlib.import_module("lutris.runners." + self.runner_name)
-        runner_class = getattr(runner_module, self.runner_name)
+        runner_class = import_runner(self.runner_name)
         runner_instance = runner_class()
         for config in ["game", "runner"]:
             for k, v in getattr(self.lutris_config, config + "_config").items():
@@ -494,7 +442,7 @@ class GameDialogCommon:
                     except Exception:
                         invalid_fields.append(option.get("label"))
         if invalid_fields:
-            ErrorDialog(_("The following fields have invalid values: ") + ", ".join(invalid_fields))
+            ErrorDialog(_("The following fields have invalid values: ") + ", ".join(invalid_fields), parent=self)
             return False
         return True
 
@@ -527,28 +475,20 @@ class GameDialogCommon:
         self.game.game_config_id = self.lutris_config.game_config_id
         self.game.runner = runner
         self.game.runner_name = self.runner_name
-        self.game.directory = runner.game_path
         self.game.is_installed = True
-        if self.runner_name in ("steam", "winesteam"):
-            self.game.steamid = self.lutris_config.game_config["appid"]
-
         self.game.config = self.lutris_config
-        self.game.save()
+        self.game.save(save_config=True)
         self.destroy()
         self.saved = True
         return True
 
     def on_custom_image_select(self, _widget, image_type):
-        dialog = Gtk.FileChooserDialog(
+        dialog = Gtk.FileChooserNative.new(
             _("Please choose a custom image"),
             self,
             Gtk.FileChooserAction.OPEN,
-            (
-                Gtk.STOCK_CANCEL,
-                Gtk.ResponseType.CANCEL,
-                Gtk.STOCK_OPEN,
-                Gtk.ResponseType.OK,
-            ),
+            None,
+            None,
         )
 
         image_filter = Gtk.FileFilter()
@@ -557,11 +497,11 @@ class GameDialogCommon:
         dialog.add_filter(image_filter)
 
         response = dialog.run()
-        if response == Gtk.ResponseType.OK:
+        if response == Gtk.ResponseType.ACCEPT:
             image_path = dialog.get_filename()
             if image_type == "banner":
                 self.game.has_custom_banner = True
-                dest_path = resources.get_banner_path(self.game.slug)
+                dest_path = os.path.join(settings.BANNER_PATH, "%s.jpg" % self.game.slug)
                 size = BANNER_SIZE
                 file_format = "jpeg"
             else:
@@ -574,18 +514,19 @@ class GameDialogCommon:
             self._set_image(image_type)
 
             if image_type == "icon":
-                resources.update_desktop_icons()
+                system.update_desktop_icons()
 
         dialog.destroy()
 
     def on_custom_image_reset_clicked(self, _widget, image_type):
         if image_type == "banner":
             self.game.has_custom_banner = False
-            dest_path = resources.get_banner_path(self.game.slug)
+            dest_path = os.path.join(settings.BANNER_PATH, "%s.jpg" % self.game.slug)
         elif image_type == "icon":
             self.game.has_custom_icon = False
             dest_path = resources.get_icon_path(self.game.slug)
         else:
             raise ValueError("Unsupported image type %s" % image_type)
-        os.remove(dest_path)
+        if os.path.isfile(dest_path):
+            os.remove(dest_path)
         self._set_image(image_type)
