@@ -1,15 +1,17 @@
 """Runner loaders"""
+from collections import defaultdict
 
 __all__ = [
     # Native
     "linux",
     "steam",
     "web",
+    "flatpak",
     # Microsoft based
     "wine",
-    "winesteam",
     "dosbox",
     # Multi-system
+    "easyrpg",
     "mame",
     "mednafen",
     "scummvm",
@@ -25,6 +27,7 @@ __all__ = [
     "snes9x",
     "mupen64plus",
     "dolphin",
+    "ryujinx",
     "yuzu",
     # Sony
     "pcsx2",
@@ -41,6 +44,7 @@ __all__ = [
     "zdoom",
 ]
 ADDON_RUNNERS = {}
+RUNNER_PLATFORMS = {}
 
 
 class InvalidRunner(Exception):
@@ -111,4 +115,14 @@ def get_runner_names():
     }
 
 
+def get_platforms():
+    """Return a dictionary of all supported platforms with their runners"""
+    platforms = defaultdict(list)
+    for runner_name in __all__:
+        runner = import_runner(runner_name)()
+        for platform in runner.platforms:
+            platforms[platform].append(runner_name)
+    return platforms
+
+
 RUNNER_NAMES = {}  # This needs to be initialized at startup with get_runner_names
