@@ -6,9 +6,8 @@ import time
 
 from lutris import runtime, settings
 from lutris.command import MonitoredCommand
-from lutris.config import LutrisConfig
 from lutris.runners import import_runner
-from lutris.util import system
+from lutris.util import linux, system
 from lutris.util.log import logger
 from lutris.util.shell import get_shell_command
 from lutris.util.strings import split_arguments
@@ -42,7 +41,7 @@ def set_regedit(
     }
     # Make temporary reg file
     reg_path = os.path.join(settings.CACHE_DIR, "winekeys.reg")
-    with open(reg_path, "w") as reg_file:
+    with open(reg_path, "w", encoding='utf-8') as reg_file:
         reg_file.write('REGEDIT4\n\n[%s]\n"%s"=%s\n' % (path, key, formatted_value[type]))
     logger.debug("Setting [%s]:%s=%s", path, key, formatted_value[type])
     set_regedit_file(reg_path, wine_path=wine_path, prefix=prefix, arch=arch)
@@ -86,6 +85,7 @@ def create_prefix(  # noqa: C901
     overrides=None,
     install_gecko=None,
     install_mono=None,
+    runner=None
 ):
     """Create a new Wine prefix."""
     # pylint: disable=too-many-locals
@@ -102,11 +102,15 @@ def create_prefix(  # noqa: C901
     # Avoid issue of 64bit Wine refusing to create win32 prefix
     # over an existing empty folder.
     if os.path.isdir(prefix) and not os.listdir(prefix):
-        os.rmdir(prefix)
+        try:
+            os.rmdir(prefix)
+        except OSError:
+            logger.error("Failed to delete %s, you may lack permissions on this folder.", prefix)
 
     if not wine_path:
-        wine = import_runner("wine")
-        wine_path = wine().get_executable()
+        if not runner:
+            runner = import_runner("wine")()
+        wine_path = runner.get_executable()
     if not wine_path:
         logger.error("Wine not found, can't create prefix")
         return
@@ -119,23 +123,27 @@ def create_prefix(  # noqa: C901
         )
         return
 
-    if install_gecko == "False":
-        overrides["mshtml"] = "disabled"
-    if install_mono == "False":
-        overrides["mscoree"] = "disabled"
-
     wineenv = {
         "WINEARCH": arch,
         "WINEPREFIX": prefix,
         "WINEDLLOVERRIDES": get_overrides_env(overrides),
+        "WINE_MONO_CACHE_DIR": os.path.join(os.path.dirname(os.path.dirname(wine_path)), "mono"),
+        "WINE_GECKO_CACHE_DIR": os.path.join(os.path.dirname(os.path.dirname(wine_path)), "gecko"),
     }
 
+    if install_gecko == "False":
+        wineenv["WINE_SKIP_GECKO_INSTALLATION"] = "1"
+        overrides["mshtml"] = "disabled"
+    if install_mono == "False":
+        wineenv["WINE_SKIP_MONO_INSTALLATION"] = "1"
+        overrides["mscoree"] = "disabled"
+
     system.execute([wineboot_path], env=wineenv)
-    for loop_index in range(50):
-        time.sleep(0.25)
+    for loop_index in range(1000):
+        time.sleep(0.5)
         if system.path_exists(os.path.join(prefix, "user.reg")):
             break
-        if loop_index == 20:
+        if loop_index == 60:
             logger.warning("Wine prefix creation is taking longer than expected...")
     if not os.path.exists(os.path.join(prefix, "user.reg")):
         logger.error("No user.reg found after prefix creation. " "Prefix might not be valid")
@@ -143,32 +151,17 @@ def create_prefix(  # noqa: C901
     logger.info("%s Prefix created in %s", arch, prefix)
     prefix_manager = WinePrefixManager(prefix)
     prefix_manager.setup_defaults()
-    if 'steamapps/common' in prefix.lower():
-        from lutris.runners.winesteam import winesteam
-        runner = winesteam()
-        logger.info("Transfering Steam information from default prefix to new prefix")
-        dest_path = '/tmp/steam.reg'
-        default_prefix = runner.get_default_prefix(runner.default_arch)
-        wineexec("regedit", args=r"/E '%s' 'HKEY_CURRENT_USER\Software\Valve\Steam'" % dest_path, prefix=default_prefix)
-        set_regedit_file(dest_path, wine_path=wine_path, prefix=prefix, arch=arch)
-        try:
-            os.remove(dest_path)
-        except FileNotFoundError:
-            logger.error("File %s was already removed", dest_path)
-        steam_drive_path = os.path.join(prefix, 'dosdevices', 's:')
-        if not system.path_exists(steam_drive_path):
-            logger.info("Linking Steam default prefix to drive S:")
-            os.symlink(os.path.join(default_prefix, 'drive_c'), steam_drive_path)
 
 
-def winekill(prefix, arch=WINE_DEFAULT_ARCH, wine_path=None, env=None, initial_pids=None):
+def winekill(prefix, arch=WINE_DEFAULT_ARCH, wine_path=None, env=None, initial_pids=None, runner=None):
     """Kill processes in Wine prefix."""
 
     initial_pids = initial_pids or []
 
     if not wine_path:
-        wine = import_runner("wine")
-        wine_path = wine().get_executable()
+        if not runner:
+            runner = import_runner("wine")()
+        wine_path = runner.get_executable()
     wine_root = os.path.dirname(wine_path)
     if not env:
         env = {"WINEARCH": arch, "WINEPREFIX": prefix}
@@ -217,6 +210,7 @@ def wineexec(  # noqa: C901
     disable_runtime=False,
     env=None,
     overrides=None,
+    runner=None
 ):
     """
     Execute a Wine command.
@@ -232,6 +226,7 @@ def wineexec(  # noqa: C901
         blocking (bool): if true, do not run the process in a thread
         config (LutrisConfig): LutrisConfig object for the process context
         watch (list): list of process names to monitor (even when in a ignore list)
+        runner (runner): the wine runner that carries the configuration to use
 
     Returns:
         Process results if the process is running in blocking mode or
@@ -248,9 +243,12 @@ def wineexec(  # noqa: C901
         include_processes = shlex.split(include_processes)
     if isinstance(exclude_processes, str):
         exclude_processes = shlex.split(exclude_processes)
+
+    if not runner:
+        runner = import_runner("wine")()
+
     if not wine_path:
-        wine = import_runner("wine")
-        wine_path = wine().get_executable()
+        wine_path = runner.get_executable()
     if not wine_path:
         raise RuntimeError("Wine is not installed")
 
@@ -267,7 +265,7 @@ def wineexec(  # noqa: C901
         arch = detect_arch(prefix, wine_path)
     if not detect_prefix_arch(prefix):
         wine_bin = winetricks_wine if winetricks_wine else wine_path
-        create_prefix(prefix, wine_path=wine_bin, arch=arch)
+        create_prefix(prefix, wine_path=wine_bin, arch=arch, runner=runner)
 
     wineenv = {"WINEARCH": arch}
     if winetricks_wine:
@@ -278,8 +276,8 @@ def wineexec(  # noqa: C901
     if prefix:
         wineenv["WINEPREFIX"] = prefix
 
-    wine_config = config or LutrisConfig(runner_slug="wine")
-    disable_runtime = disable_runtime or wine_config.system_config["disable_runtime"]
+    wine_system_config = config.system_config if config else runner.system_config
+    disable_runtime = disable_runtime or wine_system_config["disable_runtime"]
     if use_lutris_runtime(wine_path=wineenv["WINE"], force_disable=disable_runtime):
         if WINE_DIR in wine_path:
             wine_root_path = os.path.dirname(os.path.dirname(wine_path))
@@ -289,7 +287,7 @@ def wineexec(  # noqa: C901
             wine_root_path = None
         wineenv["LD_LIBRARY_PATH"] = ":".join(
             runtime.get_paths(
-                prefer_system_libs=wine_config.system_config["prefer_system_libs"],
+                prefer_system_libs=wine_system_config["prefer_system_libs"],
                 wine_path=wine_root_path,
             )
         )
@@ -297,20 +295,24 @@ def wineexec(  # noqa: C901
     if overrides:
         wineenv["WINEDLLOVERRIDES"] = get_overrides_env(overrides)
 
-    if env:
-        wineenv.update(env)
+    baseenv = runner.get_env(disable_runtime=disable_runtime)
+    baseenv.update(wineenv)
+    baseenv.update(env)
 
     command_parameters = [wine_path]
     if executable:
         command_parameters.append(executable)
     command_parameters += split_arguments(args)
+
+    runner.prelaunch()
+
     if blocking:
-        return system.execute(command_parameters, env=wineenv, cwd=working_dir)
-    wine = import_runner("wine")
+        return system.execute(command_parameters, env=baseenv, cwd=working_dir)
+
     command = MonitoredCommand(
         command_parameters,
-        runner=wine(),
-        env=wineenv,
+        runner=runner,
+        env=baseenv,
         cwd=working_dir,
         include_processes=include_processes,
         exclude_processes=exclude_processes,
@@ -331,19 +333,21 @@ def winetricks(
     config=None,
     env=None,
     disable_runtime=False,
+    system_winetricks=False,
+    runner=None
 ):
     """Execute winetricks."""
-    wine_config = config or LutrisConfig(runner_slug="wine")
     winetricks_path = os.path.join(settings.RUNTIME_DIR, "winetricks/winetricks")
-    if (wine_config.runner_config.get("system_winetricks") or not system.path_exists(winetricks_path)):
+    if system_winetricks or not system.path_exists(winetricks_path):
         winetricks_path = system.find_executable("winetricks")
         if not winetricks_path:
             raise RuntimeError("No installation of winetricks found")
     if wine_path:
         winetricks_wine = wine_path
     else:
-        wine = import_runner("wine")
-        winetricks_wine = wine().get_executable()
+        if not runner:
+            runner = import_runner("wine")()
+        winetricks_wine = runner.get_executable()
     if arch not in ("win32", "win64"):
         arch = detect_arch(prefix, winetricks_wine)
     args = app
@@ -359,28 +363,27 @@ def winetricks(
         config=config,
         env=env,
         disable_runtime=disable_runtime,
+        runner=runner
     )
 
 
-def winecfg(wine_path=None, prefix=None, arch=WINE_DEFAULT_ARCH, config=None, env=None):
+def winecfg(wine_path=None, prefix=None, arch=WINE_DEFAULT_ARCH, config=None, env=None, runner=None):
     """Execute winecfg."""
     if not wine_path:
         logger.debug("winecfg: Reverting to default wine")
         wine = import_runner("wine")
         wine_path = wine().get_executable()
 
-    winecfg_path = os.path.join(os.path.dirname(wine_path), "winecfg")
-    logger.debug("winecfg: %s", winecfg_path)
-
     return wineexec(
-        None,
+        "winecfg.exe",
         prefix=prefix,
-        winetricks_wine=winecfg_path,
-        wine_path=winecfg_path,
+        winetricks_wine=wine_path,
+        wine_path=wine_path,
         arch=arch,
         config=config,
         env=env,
         include_processes=["winecfg.exe"],
+        runner=runner
     )
 
 
@@ -408,5 +411,5 @@ def open_wine_terminal(terminal, wine_path, prefix, env):
     }
     env["WINEPREFIX"] = prefix
     shell_command = get_shell_command(prefix, env, aliases)
-    terminal = terminal or system.get_default_terminal()
-    system.execute([system.get_default_terminal(), "-e", shell_command])
+    terminal = terminal or linux.get_default_terminal()
+    system.execute([terminal, "-e", shell_command])
