@@ -1,14 +1,11 @@
 """libretro runner"""
-# Standard Library
 import os
 from gettext import gettext as _
 from operator import itemgetter
 from zipfile import ZipFile
 
-# Third Party Libraries
 import requests
 
-# Lutris Modules
 from lutris import settings
 from lutris.runners.runner import Runner
 from lutris.util import system
@@ -23,31 +20,32 @@ def get_default_config_path(path=""):
 def get_libretro_cores():
     cores = []
     runner_path = get_default_config_path()
-    if os.path.exists(runner_path):
-        # Get core identifiers from info dir
-        info_path = get_default_config_path("info")
-        if not os.path.exists(info_path):
-            req = requests.get("http://buildbot.libretro.com/assets/frontend/info.zip", allow_redirects=True)
-            if req.status_code == requests.codes.ok:  # pylint: disable=no-member
-                open(get_default_config_path('info.zip'), 'wb').write(req.content)
-                with ZipFile(get_default_config_path('info.zip'), 'r') as info_zip:
-                    info_zip.extractall(info_path)
-            else:
-                logger.error("Error retrieving libretro info archive from server: %s - %s", req.status_code, req.reason)
-        # Parse info files to fetch display name and platform/system
-        if os.path.exists(info_path):
-            for info_file in os.listdir(info_path):
-                if "_libretro.info" not in info_file:
-                    continue
-                core_identifier = info_file.replace("_libretro.info", "")
-                core_config = RetroConfig(os.path.join(info_path, info_file))
-                if "categories" in core_config.keys() and "Emulator" in core_config["categories"]:
-                    core_label = core_config["display_name"] or ""
-                    core_system = core_config["systemname"] or ""
-                    cores.append((core_label, core_identifier, core_system))
-            cores.sort(key=itemgetter(0))
-    if not cores:
-        logger.warning("No cores found")
+    if not os.path.exists(runner_path):
+        return []
+
+    # Get core identifiers from info dir
+    info_path = get_default_config_path("info")
+    if not os.path.exists(info_path):
+        req = requests.get("http://buildbot.libretro.com/assets/frontend/info.zip", allow_redirects=True)
+        if req.status_code == requests.codes.ok:  # pylint: disable=no-member
+            with open(get_default_config_path('info.zip'), 'wb') as info_zip:
+                info_zip.write(req.content)
+            with ZipFile(get_default_config_path('info.zip'), 'r') as info_zip:
+                info_zip.extractall(info_path)
+        else:
+            logger.error("Error retrieving libretro info archive from server: %s - %s", req.status_code, req.reason)
+            return []
+    # Parse info files to fetch display name and platform/system
+    for info_file in os.listdir(info_path):
+        if "_libretro.info" not in info_file:
+            continue
+        core_identifier = info_file.replace("_libretro.info", "")
+        core_config = RetroConfig(os.path.join(info_path, info_file))
+        if "categories" in core_config.keys() and "Emulator" in core_config["categories"]:
+            core_label = core_config["display_name"] or ""
+            core_system = core_config["systemname"] or ""
+            cores.append((core_label, core_identifier, core_system))
+    cores.sort(key=itemgetter(0))
     return cores
 
 
@@ -109,14 +107,24 @@ class libretro(Runner):
 
     def get_platform(self):
         game_core = self.game_config.get("core")
-        if game_core:
-            for core in LIBRETRO_CORES:
-                if core[1] == game_core:
-                    return core[2]
+        if not game_core:
+            logger.warning("Game don't have a core set")
+            return
+        for core in LIBRETRO_CORES:
+            if core[1] == game_core:
+                return core[2]
+        logger.warning("'%s' not found in Libretro cores", game_core)
         return ""
 
     def get_core_path(self, core):
-        return os.path.join(settings.RUNNER_DIR, "retroarch", "cores", "{}_libretro.so".format(core))
+        """Return the path of a core, prioritizing Retroarch cores"""
+        lutris_cores_folder = os.path.join(settings.RUNNER_DIR, "retroarch", "cores")
+        retroarch_core_folder = os.path.join(os.path.expanduser("~/.config/retroarch/cores"))
+        core_filename = "{}_libretro.so".format(core)
+        retroarch_core = os.path.join(retroarch_core_folder, core_filename)
+        if system.path_exists(retroarch_core):
+            return retroarch_core
+        return os.path.join(lutris_cores_folder, core_filename)
 
     def get_version(self, use_default=True):
         return self.game_config["core"]
@@ -125,27 +133,27 @@ class libretro(Runner):
         return system.path_exists(self.get_executable())
 
     def is_installed(self, core=None):
-        if self.game_config.get("core") and core is None:
+        if not core and self.config and self.game_config.get("core"):
             core = self.game_config["core"]
         if not core or self.runner_config.get("runner_executable"):
             return self.is_retroarch_installed()
-
         is_core_installed = system.path_exists(self.get_core_path(core))
         return self.is_retroarch_installed() and is_core_installed
 
     def install(self, version=None, downloader=None, callback=None):
+        captured_super = super()  # super() does not work inside install_core()
 
         def install_core():
             if not version:
                 if callback:
                     callback()
             else:
-                super(libretro, self).install(version, downloader, callback)
+                captured_super.install(version, downloader, callback)
 
         if not self.is_retroarch_installed():
-            super(libretro, self).install(version=None, downloader=downloader, callback=install_core)
+            captured_super.install(version=None, downloader=downloader, callback=install_core)
         else:
-            super(libretro, self).install(version, downloader, callback)
+            captured_super.install(version, downloader, callback)
 
     def get_run_data(self):
         return {
@@ -170,9 +178,9 @@ class libretro(Runner):
         # TODO: review later
         # Create retroarch.cfg if it doesn't exist.
         if not system.path_exists(config_file):
-            f = open(config_file, "w")
-            f.write("# Lutris RetroArch Configuration")
-            f.close()
+            with open(config_file, "w", encoding='utf-8') as f:
+                f.write("# Lutris RetroArch Configuration")
+                f.close()
 
             # Build the default config settings.
             retro_config = RetroConfig(config_file)
@@ -233,8 +241,6 @@ class libretro(Runner):
                 # TODO Add dialog for copying the firmware in the correct
                 # location
 
-        return True
-
     def get_runner_parameters(self):
         parameters = []
 
@@ -281,11 +287,11 @@ class libretro(Runner):
     # Checks whether the retroarch or libretro directories can be uninstalled.
     def can_uninstall(self):
         retroarch_path = os.path.join(settings.RUNNER_DIR, 'retroarch')
-        return os.path.isdir(retroarch_path) or super(libretro, self).can_uninstall()
+        return os.path.isdir(retroarch_path) or super().can_uninstall()
 
     # Remove the `retroarch` directory.
     def uninstall(self):
         retroarch_path = os.path.join(settings.RUNNER_DIR, 'retroarch')
         if os.path.isdir(retroarch_path):
             system.remove_folder(retroarch_path)
-        super(libretro, self).uninstall()
+        super().uninstall()
