@@ -1,14 +1,13 @@
 """Run web based games"""
-# Standard Library
 import os
 import string
 from gettext import gettext as _
 from urllib.parse import urlparse
 
-# Lutris Modules
-from lutris import pga, settings
+from lutris import settings
+from lutris.database.games import get_game_by_field
 from lutris.runners.runner import Runner
-from lutris.util import datapath, resources, system
+from lutris.util import datapath, linux, resources, system
 from lutris.util.strings import split_arguments
 
 DEFAULT_ICON = os.path.join(datapath.get(), "media/default_icon.png")
@@ -122,6 +121,14 @@ class web(Runner):
             "default": False,
             "help": _("Enable Adobe Flash Player."),
         },
+        {
+            "option": "user_agent",
+            "label": _("Custom User-Agent"),
+            "type": "string",
+            "default": "",
+            "help": _("Overrides the default User-Agent header used by the runner."),
+            "advanced": True,
+        },
         {
             "option": "devtools",
             "label": _("Debug with Developer Tools"),
@@ -168,8 +175,8 @@ class web(Runner):
     system_options_override = [{"option": "disable_runtime", "default": True}]
     runner_executable = "web/electron/electron"
 
-    def get_env(self, os_env=True):
-        env = super(web, self).get_env(os_env)
+    def get_env(self, os_env=True, disable_runtime=False):
+        env = super().get_env(os_env, disable_runtime=disable_runtime)
 
         enable_flash_player = self.runner_config.get("enable_flash")
         env["ENABLE_FLASH_PLAYER"] = "1" if enable_flash_player else "0"
@@ -197,7 +204,7 @@ class web(Runner):
                 }
             url = "file://" + url
 
-        game_data = pga.get_game_by_field(self.config.game_config_id, "configpath")
+        game_data = get_game_by_field(self.config.game_config_id, "configpath")
 
         # keep the old behavior from browser runner, but with support for extra arguments!
         if self.runner_config.get("external_browser"):
@@ -248,5 +255,10 @@ class web(Runner):
         if self.runner_config.get("window_size"):
             command.append("--window-size")
             command.append(self.runner_config.get("window_size"))
+        if self.runner_config.get("user_agent"):
+            command.append("--user-agent")
+            command.append(self.runner_config.get("user_agent"))
+        if linux.LINUX_SYSTEM.is_flatpak:
+            command.append("--no-sandbox")
 
         return {"command": command, "env": self.get_env(False)}
