@@ -7,8 +7,7 @@ from gettext import gettext as _
 
 from lutris import runners
 from lutris.util import linux, system
-from lutris.util.display import DISPLAY_MANAGER, SCREEN_SAVER_INHIBITOR, USE_DRI_PRIME
-from lutris.util.graphics import drivers
+from lutris.util.display import DISPLAY_MANAGER, SCREEN_SAVER_INHIBITOR, USE_DRI_PRIME, has_graphic_adapter_description
 
 VULKAN_DATA_DIRS = [
     "/usr/local/etc/vulkan",  # standard site-local location
@@ -31,6 +30,24 @@ def get_resolution_choices():
     return resolution_choices
 
 
+def get_locale_choices():
+    """Return list of available locales as label, value tuples
+    suitable for inclusion in drop-downs.
+    """
+    locales = system.get_locale_list()
+
+    # adds "(recommended)" string to utf8 locales
+    locales_humanized = locales.copy()
+    for index, locale in enumerate(locales_humanized):
+        if "utf8" in locale:
+            locales_humanized[index] += " " + _("(recommended)")
+
+    locale_choices = list(zip(locales_humanized, locales))
+    locale_choices.insert(0, (_("System"), ""))
+
+    return locale_choices
+
+
 def get_output_choices():
     """Return list of outputs for drop-downs"""
     displays = DISPLAY_MANAGER.get_display_names()
@@ -65,15 +82,13 @@ def get_optirun_choices():
     return choices
 
 
-def get_gpu_vendor_cmd(nvidia_files):
+def get_gpu_vendor_cmd(is_nvidia):
     """Run glxinfo command to get vendor based on certain conditions"""
-    glxinfocmd = "glxinfo | grep -i opengl | grep -i vendor"
-
-    if USE_DRI_PRIME == 1:
-        glxinfocmd = "DRI_PRIME=1 glxinfo | grep -i opengl | grep -i vendor"
-    elif nvidia_files == 1:
-        glxinfocmd = "__GLX_VENDOR_LIBRARY_NAME=nvidia glxinfo | grep -i opengl | grep -i vendor"
-    return glxinfocmd
+    if is_nvidia:
+        return "__GLX_VENDOR_LIBRARY_NAME=nvidia glxinfo | grep -i opengl | grep -i vendor"
+    if USE_DRI_PRIME:
+        return "DRI_PRIME=1 glxinfo | grep -i opengl | grep -i vendor"
+    return "glxinfo | grep -i opengl | grep -i vendor"
 
 
 def get_vk_icd_choices():
@@ -109,19 +124,30 @@ def get_vk_icd_choices():
     amdvlk_files = ":".join(amdvlk)
     amdvlkpro_files = ":".join(amdvlkpro)
 
-    glxinfocmd = get_gpu_vendor_cmd(0)
-    if nvidia_files:
-        glxinfocmd = get_gpu_vendor_cmd(1)
+    intel_name = _("Auto: Intel Open Source (MESA: ANV)")
+    amdradv_name = _("Auto: AMD RADV Open Source (MESA: RADV)")
+    nvidia_name = _("Auto: Nvidia Proprietary")
+
+    glxinfocmd = get_gpu_vendor_cmd(bool(nvidia_files))
     with subprocess.Popen(glxinfocmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT) as glxvendorget:
         glxvendor = glxvendorget.communicate()[0].decode("utf-8")
     default_gpu = glxvendor
 
     if "Intel" in default_gpu:
-        choices = [(_("Auto: Intel Open Source (MESA: ANV)"), intel_files)]
+        choices = [(intel_name, intel_files)]
     elif "AMD" in default_gpu:
-        choices = [(_("Auto: AMD RADV Open Source (MESA: RADV)"), amdradv_files)]
+        choices = [(amdradv_name, amdradv_files)]
     elif "NVIDIA" in default_gpu:
-        choices = [(_("Auto: Nvidia Proprietary"), nvidia_files)]
+        choices = [(nvidia_name, nvidia_files)]
+    elif USE_DRI_PRIME:
+        # We have multiple video chipsets, pick something that is instlaled if possible;
+        # we prefer NVIDIA and AMD over Intel, because don't we all?
+        if bool(nvidia_files) and has_graphic_adapter_description("NVIDIA"):
+            choices = [(nvidia_name, nvidia_files)]
+        elif bool(amdradv_files) and has_graphic_adapter_description("AMD"):
+            choices = [(amdradv_name, amdradv_files)]
+        elif bool(intel_files) and has_graphic_adapter_description("Intel"):
+            choices = [(intel_name, intel_files)]
 
     if intel_files:
         choices.append(("Intel Open Source (MESA: ANV)", intel_files))
@@ -184,7 +210,7 @@ system_options = [  # pylint: disable=invalid-name
         "label": _("Enable gamescope"),
         "default": False,
         "advanced": True,
-        "condition": bool(system.find_executable("gamescope")) and not drivers.is_nvidia(),
+        "condition": bool(system.find_executable("gamescope")) and linux.LINUX_SYSTEM.nvidia_gamescope_support(),
         "help": _("Use gamescope to draw the game window isolated from your desktop.\n"
                   "Use Ctrl+Super+F to toggle fullscreen"),
     },
@@ -406,6 +432,17 @@ system_options = [  # pylint: disable=invalid-name
                   "Choose from the list of detected terminal apps or enter "
                   "the terminal's command or path."),
     },
+    {
+        "option": "locale",
+        "type": "choice",
+        "label": _("Locale"),
+        "choices": (
+            get_locale_choices()
+        ),
+        "default": "",
+        "advanced": False,
+        "help": _("Can be used to force certain locale for an app. Fixes encoding issues in legacy software."),
+    },
     {
         "option": "env",
         "type": "mapping",
